<?php
/**
 * Theme functions and definitions.
 *
 * @package StagFramework
 * @subpackage GeekLove
 */

/**
 * Set Max Content Width.
 *
 * @var integer
 */
if ( ! isset( $content_width ) ) {
	$content_width = 940;
}

/**
 * Set cookie for Retina displays.
 *
 * @var bool
 */
global $is_retina;
( isset( $_COOKIE['retina'] ) ) ? $is_retina = true : $is_retina = false;

/* Theme Setup */
if ( ! function_exists( 'stag_theme_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function is hooked into the after_setup_theme hook, which
	 * runs before the init hook. The init hook is too late for some features, such
	 * as indicating support post thumbnails.
	 */
	function stag_theme_setup() {
		/**
		 * Makes theme available for translation.
		 *
		 * Translations can be added to the /languages/ directory.
		 */
		load_theme_textdomain( 'geeklove', get_template_directory() . '/languages' );

		// This theme uses wp_nav_menu() in following locations.
		register_nav_menu( 'primary-menu', __( 'Primary Menu', 'geeklove' ) );

		/**
		 * Enable support for Post Thumbnails on posts and pages
		 *
		 * @link http://codex.wordpress.org/Function_Reference/add_theme_support#Post_Thumbnails
		 */
		add_theme_support( 'post-thumbnails' );
		add_theme_support( 'automatic-feed-links' );

		/**
		 * Add support for title tag.
		 *
		 * @since 2.2.0
		 */
		add_theme_support( 'title-tag' );

		/**
		 * Add theme support for selective refresh for widgets.
		 *
		 * @since 2.2.0
		 */
		add_theme_support( 'customize-selective-refresh-widgets' );

		set_post_thumbnail_size( 170, 160 ); // Normal post thumbnails.

		add_image_size( 'gallery-thumb', 460, 310, true ); // Photo Gallery Thumbnails.

		/**
		* Add support for Gutenberg.
		*/
		add_theme_support(
			'editor-color-palette', array(
				array(
					'name'  => __( 'Abbey', 'geeklove' ),
					'slug'  => 'abbey',
					'color' => '#494B51',
				),
				array(
					'name'  => __( 'Oslo Gray', 'geeklove' ),
					'slug'  => 'oslo-gray',
					'color' => '#859093',
				),
				array(
					'color' => geeklove_get_thememod_value( 'accent-color' ),
				),
				array(
					'color' => geeklove_get_thememod_value( 'style_background_color' ),
				),
			)
		);

		add_theme_support( 'align-wide' );
		add_theme_support( 'responsive-embeds' );
		add_theme_support( 'wp-block-styles' );
		add_theme_support( 'editor-styles' );
		add_editor_style( '/assets/css/editor-style.css' );

		// Upgrade theme settings to v3, if necessary.
		geeklove_theme_upgrade();
	}
endif; // stag_theme_setup.
add_action( 'after_setup_theme', 'stag_theme_setup' );

/**
 * Register theme sidebar areas.
 *
 * @return void
 */
function stag_sidebar_init() {

	register_sidebar(
		array(
			'name'          => __( 'Footer Widgets', 'geeklove' ),
			'id'            => 'sidebar-footer',
			'before_widget' => '<div class="grid-4 %2$s">',
			'after_widget'  => '</div>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);
}
add_action( 'widgets_init', 'stag_sidebar_init' );

/**
 * Enqueue editor styles for Gutenberg
 */
function geeklove_block_editor_styles() {

	/**
	 * Getting & Setting Google Fonts
	 */
	$fonts = geeklove_get_google_font_uri();
	if ( '' !== $fonts ) {
		// Enqueue the fonts.
		wp_enqueue_style( 'geeklove-google-fonts', $fonts, array(), STAG_THEME_VERSION );
	}

	// Block Editor Styles.
	wp_enqueue_style( 'fonts', get_template_directory_uri() . '/assets/css/fonts.css', array(), STAG_THEME_VERSION );

	$font_header = geeklove_get_thememod_value( 'font-headers' );
	$font_body   = geeklove_get_thememod_value( 'font-body' );

	$accent_color   = geeklove_get_thememod_value( 'accent-color' );
	$style_bg_color = geeklove_get_thememod_value( 'style_background_color' );
	$text_color     = geeklove_get_thememod_value( 'text-color' );

	wp_add_inline_style(
		'geeklove-google-fonts',
		".editor-styles-wrapper {
			--accent-color: {$accent_color};
			--style-background-color: {$style_bg_color};
			--text-color: {$text_color};
			--font-body: '{$font_body}';
			--font-header: '{$font_header}';
		}"
	);
}
add_action( 'enqueue_block_editor_assets', 'geeklove_block_editor_styles' );


/**
 * Enqueues scripts and styles for front end.
 *
 * @return void
 */
function stag_enqueue_scripts() {
	global $is_IE;
	/** Register Styles */
	wp_register_style( 'lightbox', get_template_directory_uri() . '/assets/css/lightbox.css', '', '2.7.1', 'screen' );

	// Google fonts.
	$google_request = geeklove_get_google_font_uri();

	if ( '' !== $google_request ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'geeklove-google-fonts',
			$google_request,
			'',
			STAG_THEME_VERSION
		);
	}

	/** Enqueue Styles */
	wp_enqueue_style( 'stag-style', get_stylesheet_uri(), '', STAG_THEME_VERSION );
	wp_dequeue_style( 'stag-shortcode-styles' );
	wp_enqueue_style( 'shortcode-styles', get_template_directory_uri() . '/assets/css/shortcodes.css', '', STAG_THEME_VERSION );
	wp_enqueue_style( 'fonts', get_template_directory_uri() . '/assets/css/fonts.css' );

	// IE CSS.
	global $is_IE;
	if ( $is_IE ) {
		wp_enqueue_style( 'ie8', get_template_directory_uri() . '/assets/css/ie.css' );
	}

	$js_dir = get_template_directory_uri() . '/assets/js/';

	/** Register Scripts */
	wp_register_script( 'script', $js_dir . 'jquery.custom.js', array( 'jquery' ), STAG_THEME_VERSION, true );
	wp_register_script( 'stag-plugins', $js_dir . 'plugins.js', array( 'jquery' ), STAG_THEME_VERSION, true );
	wp_register_script( 'isotope', $js_dir . 'jquery.isotope.js', array( 'jquery', 'imagesloaded' ), '3.0.4', true );
	wp_register_script( 'lightbox', $js_dir . 'lightbox.js', array( 'jquery' ), '2.7.1', true );
	wp_register_script( 'backstretch', $js_dir . 'jquery.backstretch.min.js', array( 'jquery' ), '2.1.15', true );
	wp_register_script( 'countdown', $js_dir . 'jquery.countdown.js', array( 'jquery' ), '2.0.2', true );
	wp_register_script( 'dropdown', $js_dir . 'jquery.dropdown.js', array( 'jquery' ), STAG_THEME_VERSION, true );
	wp_register_script( 'css3-mediaqueries', $js_dir . 'css3-mediaqueries.min.js', array(), '1.0.0', false );
	wp_register_script( 'selectivizr', $js_dir . 'selectivizr-min.js', array(), '1.0.2', false );
	wp_register_script( 'html5shiv', $js_dir . 'html5shiv.js', array(), '3.7.3', false );

	/** Enqueue Scripts */
	wp_enqueue_script( 'jquery' );
	wp_enqueue_script( 'script' );
	wp_enqueue_script( 'stag-plugins' );

	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' ); // loads the javascript required for threaded comments
	}

	if (
		get_page_template_slug() == ( 'widgetized.php' || 'page-templates/template-photo-gallery.php' )
		|| ( is_single() && 'gallery' === get_post_type() )
		) {

		wp_enqueue_script( 'isotope' );

		if ( ! wp_is_mobile() ) {
			wp_enqueue_script( 'lightbox' );
			wp_enqueue_style( 'lightbox' );
		}
	}

	if ( is_page_template( 'widgetized.php' ) ) {
		wp_enqueue_script( 'backstretch' );
		wp_enqueue_script( 'countdown' );
		wp_enqueue_script( 'dropdown' );
	}

	/**
	 * Localize strings used in jQuery Countdown.
	 *
	 * @since 1.2.5
	 */
	wp_localize_script(
		'stag-plugins', 'countdownVars', array(
			'labels'  => array(
				__( 'Years', 'geeklove' ),
				__( 'Months', 'geeklove' ),
				__( 'Weeks', 'geeklove' ),
				__( 'Days', 'geeklove' ),
				__( 'Hours', 'geeklove' ),
				__( 'Minutes', 'geeklove' ),
				__( 'Seconds', 'geeklove' ),
			),
			'labels1' => array(
				__( 'Year', 'geeklove' ),
				__( 'Month', 'geeklove' ),
				__( 'Week', 'geeklove' ),
				__( 'Day', 'geeklove' ),
				__( 'Hour', 'geeklove' ),
				__( 'Minute', 'geeklove' ),
				__( 'Second', 'geeklove' ),
			),
		)
	);

	wp_localize_script(
		'script', 'stag_vars', array(
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
		)
	);

	if ( $is_IE ) {
		wp_enqueue_script( 'css3-mediaqueries' );
		wp_enqueue_script( 'selectivizr' );
		wp_enqueue_script( 'html5shiv' );
	}
}
add_action( 'wp_enqueue_scripts', 'stag_enqueue_scripts' );

/**
 * Blog Pagination.
 *
 * @return mixed
 */
function pagination() {
	global $wp_query;

	$total_pages = $wp_query->max_num_pages;
	$big         = 999999999;

	if ( $total_pages > 1 ) {
		$current_page = max( 1, get_query_var( 'paged' ) );

		$return = paginate_links(
			array(
				'base'      => str_replace( $big, '%#%', esc_url( get_pagenum_link( $big ) ) ),
				'format'    => '/page/%#%',
				'current'   => $current_page,
				'total'     => $total_pages,
				'prev_next' => false,
			)
		);

		echo "<div class='pages'>{$return}</div>";
	} else {
		return false;
	}
}

/**
 * Template for comments and pingbacks.
 *
 * Used as a callback by wp_list_comments() for displaying the comments.
 *
 * @return void
 */
function stag_comments( $comment, $args, $depth ) {

	$isByAuthor = false;

	if ( $comment->comment_author_email == get_the_author_meta( 'email' ) ) {
		$isByAuthor = true;
	}

	$GLOBALS['comment'] = $comment; ?>
	<li <?php comment_class(); ?> id="li-comment-<?php comment_ID(); ?>">
		<div id="comment-<?php comment_ID(); ?>" class="the-comment">

			<div class="comment-body clearfix">
				<div class="avatar-wrap">
					<?php
					global $is_retina;
					if ( $is_retina ) {
						echo get_avatar( $comment, $size = '112' );
					} else {
						echo get_avatar( $comment, $size = '56' );
					}
					?>
				</div>
				<div class="comment-area">
					<?php
					comment_reply_link(
						array_merge(
							$args, array(
								'depth'     => $depth,
								'max_depth' => $args['max_depth'],
							)
						)
					);
					?>
					<h3 class="comment-author"><?php echo get_comment_author_link(); ?></h3>
					<p class="comment-date"><?php echo human_time_diff( get_comment_time( 'U' ), current_time( 'timestamp' ) ) . ' ' . __( 'ago', 'geeklove' ); ?></p>
					<?php if ( $comment->comment_approved == '0' ) : ?>
						<em class="moderation"><?php _e( 'Your comment is awaiting moderation.', 'geeklove' ); ?></em>
					<?php endif; ?>
					<div class="comment-text">
						<?php comment_text(); ?>
					</div>
				</div>
			</div>

		</div>
	</li>

	<?php
}

function stag_list_pings( $comment, $args, $depth ) {
	$GLOBALS['comment'] = $comment;
	?>
	<li id="comment-<?php comment_ID(); ?>"><?php comment_author_link(); ?>
	<?php
}


if ( ! function_exists( 'custom_excerpt_length' ) ) {
	function custom_excerpt_length( $length ) {
		return geeklove_get_thememod_value( 'geeklove_post_excerpt_length' );
	}
	add_filter( 'excerpt_length', 'custom_excerpt_length', 999 );
}

if ( ! function_exists( 'new_excerpt_more' ) ) {
	function new_excerpt_more( $more ) {
		global $post;
		return ' <a class="read-more" data-through="gateway" data-postid="' . $post->ID . '" href="' . get_permalink( $post->ID ) . '">' . geeklove_get_thememod_value( 'geeklove_post_excerpt_text' ) . '</a>';
	}
	add_filter( 'excerpt_more', 'new_excerpt_more' );
}

/**
 * Get Wedding Date and Day
 */
function stag_wedding_date( $arg, $data = null ) {
	if ( $data === null ) {
		$date = geeklove_get_thememod_value( 'geeklove_wedding_date' );
	} else {
		$date = $data;
	}

	return date_i18n( $arg, strtotime( $date ) );
}

/**
 * Retrieve wedding date.
 *
 * @return mixed
 */
function stag_full_wedding_date() {
	return sprintf(
		'<span class="day-date accent-color">%1$s</span> <span class="month">%2$s</span> <span class="year accent-color">%3$s</span>',
		stag_wedding_date( 'l d' ),
		stag_wedding_date( 'F' ),
		stag_wedding_date( 'Y' )
	);
}

function stag_full_date( $date ) {
	return stag_wedding_date( 'l d F Y', $date );
}

/**
 * Get All available events
 *
 * @return array An array containing Events titles.
 */
function stag_all_wedding_events() {
	$query  = new WP_Query( 'post_type=events&posts_per_page=-1' );
	$titles = array();

	if ( $query->have_posts() ) :
		while ( $query->have_posts() ) :
			$query->the_post();
			$titles[ sanitize_title_with_dashes( get_the_title() ) ] = get_the_title();
		endwhile;
	endif;
	wp_reset_postdata();

	return $titles;
}


/**
 * Check if string starts with a particular charcter
 */
function startsWith( $haystack, $needle ) {
	return ! strncmp( $haystack, $needle, strlen( $needle ) );
}

function get_attachment_id_from_src( $image_src ) {
	global $wpdb;
	$query = "SELECT ID FROM {$wpdb->posts} WHERE guid='$image_src'";
	$id    = $wpdb->get_var( $query );
	return $id;
}

/**
 * Register the required plugins for this theme.
 *
 * In this example, we register two plugins - one included with the TGMPA library
 * and one from the .org repo.
 *
 * The variable passed to tgmpa_register_plugins() should be an array of plugin
 * arrays.
 *
 * This function is hooked into tgmpa_init, which is fired within the
 * TGM_Plugin_Activation class constructor.
 *
 * @since 1.4
 */
function geeklove_required_plugins() {
	$plugins = array(
		// This is an example of how to include a plugin from the WordPress Plugin Repository.
		array(
			'name'     => __( 'StagTools', 'geeklove' ),
			'slug'     => 'stagtools',
			'required' => true,
		),
		array(
			'name'     => __( 'Stag Custom Sidebar', 'geeklove' ),
			'slug'     => 'stag-custom-sidebars',
			'required' => false,
		),
		array(
			'name'             => __( 'Geeklove Assistant', 'geeklove' ),
			'slug'             => 'geeklove-assistant',
			'source'           => get_parent_theme_file_path() . '/assistant/geeklove-assistant-1.0.1.zip',
			'required'         => true,
			'force_activation' => true,
			'version'          => '1.0.1',
			'external_url'     => 'https://github.com/codestag/geeklove-assistant/',
		),
		array(
			'name'         => __( 'Envato Market', 'geeklove' ),
			'slug'         => 'wp-envato-market',
			'source'       => esc_url( 'https://envato.github.io/wp-envato-market/dist/envato-market.zip' ),
			'required'     => false,
			'version'      => '2.0.1',
			'external_url' => esc_url( 'https://github.com/envato/wp-envato-market' ),
		),
	);

	tgmpa( $plugins );
}
add_action( 'tgmpa_register', 'geeklove_required_plugins' );

/**
 * RSVP Widget ajax functionality.
 *
 * @since 1.2.8
 * @return void
 */
function stag_rsvp_widget_form_ajax() {
	if ( wp_verify_nonce( $_POST['_wpnonce'], 'new-post' ) ) {
		$post = array(
			'post_title'  => $_POST['attendee_name'],
			'post_status' => 'publish',
			'post_type'   => 'rsvp',
		);

		$post_id = wp_insert_post( $post );

		$results = array();

		if ( $post_id != '' ) {
			if ( $_POST['attendee_event'] == 'no-events' ) {
				$_POST['attendee_number'] = 0;
			}
			add_post_meta( $post_id, '_stag_attendee_number', $_POST['attendee_number'] );
			add_post_meta( $post_id, '_stag_attendee_event', $_POST['attendee_event'] );

			if ( 'no-events' == $_POST['attendee_event'] ) {
				$results['result'] = false;
			} else {
				$results['result'] = true;
			}

			wp_send_json_success( $results );
		}
	} else {
		header( 'HTTP/1.0 400 Bad error' );
	}

	die();
}
add_action( 'wp_ajax_rsvp_widget', 'stag_rsvp_widget_form_ajax' );
add_action( 'wp_ajax_nopriv_rsvp_widget', 'stag_rsvp_widget_form_ajax' );

/**
 * Add custom css into head.
 *
 * @since 2.0
 */
function stag_display_custom_css() {
	ob_start();
	$output = '';
	echo apply_filters( 'stag_custom_styles', $output );
	$css = ob_get_clean();

	echo "<style type='text/css'>$css</style>" . "\n";
}
add_action( 'wp_head', 'stag_display_custom_css' );

/**
 * Check if there is any third party plugin active
 *
 * @since 1.1
 */
function stag_check_third_party_seo() {
	if ( class_exists( 'HeadSpace2_Admin' ) ) {
		return true;
	}
	if ( defined( 'AIOSEOP_VERSION' ) ) {
		return true;
	}
	if ( defined( 'WPSEO_VERSION' ) ) {
		return true;
	}

	return false;
}

if ( ! function_exists( 'geeklove_assistant_notice' ) ) {
	/**
	 * Alerts if assistant plugin is not activated.
	 *
	 * @since 3.1.0
	 * @return void
	 */
	function geeklove_assistant_notice() {
		if ( ! class_exists( 'Geeklove_Assistant' ) && current_user_can( 'activate_plugins' ) ) {
			$page_link = self_admin_url( 'themes.php?page=tgmpa-install-plugins' );
			$message       = '<p>' . sprintf( '%1$s<br><strong>%2$s<a href="%3$s">this page</a>.', __('As per ThemeForest guidelines, some functionalities are now excluded from theme and moved to an assistant plugin.', 'geeklove' ), __( 'To use Geeklove with full features, please activate Geeklove Assistant from ', 'geeklove' ), esc_url( $page_link ) ) . '</strong></p>';

			echo '<div class="error">' . $message . '</div>'; // phpcs:ignore
		}
	}

}
add_action( 'admin_notices', 'geeklove_assistant_notice' );

/**
 * Include framework and other files
 */
/**
 * Custom template tags for this theme.
 */
require_once get_template_directory() . '/framework/stag-framework.php';
require_once get_template_directory() . '/includes/customizer/load.php';
require_once get_template_directory() . '/includes/template-tags.php';
require_once get_template_directory() . '/config-gravityforms/config.php';

require_once get_template_directory() . '/includes/upgrade.php';

/**
 * Include theme updater file.
 *
 * @since 3.0.0
 */
require_once get_template_directory() . '/includes/updater/theme-updater.php';
