<?php
/**
 * Handle theme upgrade process.
 *
 * @package GeekLove
 * @since 3.0.0
 */

if ( ! function_exists( 'geeklove_theme_upgrade' ) ) :
	/**
	 * Handle theme upgrade.
	 *
	 * @since 3.0.0.
	 * @return bool|void
	 */
	function geeklove_theme_upgrade() {
		// Grab old theme values.
		$framework_options = get_option( 'stag_framework_values' );

		// Bail early if old framework values are not found.
		if ( ! is_array( $framework_options ) ) return;

		$legacy_options = array_filter( $framework_options );
		$has_updated    = get_theme_mod( 'has_geeklove_migrated' );

		if ( ! $has_updated && $legacy_options && '' !== $legacy_options['settings_updated'] ) {
			$mapped_settings = geeklove_mapped_settings();

			foreach ( $legacy_options as $legacy_key => $legacy_value ) {
				if ( array_key_exists( $legacy_key, $mapped_settings ) ) {
					$filtered_value = geeklove_filter_legacy_value( $legacy_key, $legacy_value );

					set_theme_mod( $mapped_settings[ $legacy_key ], $filtered_value );
				}
			}

			// @todo Remove `stag_framework_values` option in a future patch later v3.0.0.
			add_action( 'admin_notices', 'geeklove_upgrade_notice' );
			set_theme_mod( 'has_geeklove_migrated', true );
		}
	}
endif; // End of geeklove_theme_upgrade.

/**
 * Map old theme key settings to new.
 *
 * @since 3.0.0
 * @return array
 */
function geeklove_mapped_settings() {
	$settings = [
		'general_text_logo'             => 'geeklove_text_logo',
		'general_custom_logo'           => 'geeklove_custom_logo',
		'general_contact_email'         => 'geeklove_contact_email',
		'general_tracking_code'         => 'geeklove_tracking_code',
		'general_post_excerpt_length'   => 'geeklove_post_excerpt_length',
		'general_guestbook_count'       => 'geeklove_guestbook_count',
		'general_post_excerpt_text'     => 'geeklove_post_excerpt_text',
		'general_disable_seo_settings'  => 'geeklove_disable_seo_settings',
		'style_background_color'        => 'style_background_color',
		'accent_color'                  => 'accent-color',
		'style_body_font'               => 'font-body',
		'style_heading_font'            => 'font-header',
		'style_font_script'             => 'google-font-subset',
		'style_custom_css'              => 'custom_css',
		'wedding_date'                  => 'geeklove_wedding_date',
		'wedding_time'                  => 'geeklove_wedding_time',
		'wedding_bridegroom_first_name' => 'geeklove_wedding_bridegroom_first_name',
		'wedding_bridegroom_last_name'  => 'geeklove_wedding_bridegroom_last_name',
		'wedding_bridegroom_avatar'     => 'geeklove_wedding_bridegroom_avatar',
		'wedding_bridegroom_bio'        => 'geeklove_wedding_bridegroom_bio',
		'wedding_bride_first_name'      => 'geeklove_wedding_bride_first_name',
		'wedding_bride_last_name'       => 'geeklove_wedding_bride_last_name',
		'wedding_bride_avatar'          => 'geeklove_wedding_bride_avatar',
		'wedding_bride_bio'             => 'geeklove_wedding_bride_bio',
		'wedding_slideshow'             => 'geeklove_wedding_slideshow',
		'wedding_slideshow_duration'    => 'geeklove_wedding_slideshow_duration',
		'wedding_fade_duration'         => 'geeklove_wedding_fade_duration',
		'blog_cover_image'              => 'geeklove_blog_cover_image',
		'blog_title'                    => 'geeklove_blog_title',
		'blog_subtitle'                 => 'geeklove_blog_subtitle',
		'blog_footer'                   => 'geeklove_site_footer',
	];

	return $settings;
}

/**
 * Filter/sanitize legacy values before saving into customizer.
 *
 * @param string $key The settings ID.
 * @param string $value The setting value to sanitize.
 * @return mixed
 */
function geeklove_filter_legacy_value( $key, $value ) {
	$filtered_value = '';
	$old_settings   = get_option( 'stag_framework_values' );
	$original_value = $old_settings[ $key ];

	switch ( $key ) {
		case 'style_body_font':
		case 'style_header_font':
			$font           = explode( ':', $value );
			$filtered_value = $font[0];
			break;

		case 'general_disable_seo_settings':
			$filtered_value = ( 'off' === $value ) ? false : true;
			break;

		case 'style_font_script':
			$filtered_value = 14;
			break;

		case 'blog_footer':
			$filtered_value = stripslashes( $value );
			break;

		default:
			$filtered_value = $value;
			break;
	}

	return $filtered_value;
}

/**
 * Add admin notice info for new settings location.
 *
 * @since 3.0.0
 * @return void
 */
function geeklove_upgrade_notice() {
	$class   = 'notice notice-info is-dismissible';
	$message = sprintf(
		/* translators: %s: Link to customizer. */
		__( 'Looking for theme settings? It has been moved under Appearance &rarr; <a href="%s">Customize</a>.', 'geeklove' ),
		admin_url( 'customize.php' )
	);

	printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $message ); // WPCS: XSS ok.
}
